#include "findfile.h"
#include "ui_form.h"

FindFile::FindFile(QWidget *parent)
    : QDialog(parent), ui(new Ui::FindFile)
{
    ui->setupUi(this);
    ui->directoryComboBox->addItem(QDir::currentPath());
}

FindFile::~FindFile()
{
    delete ui;
}

void FindFile::on_browseButton_clicked()
{
    QString directory = QFileDialog::getExistingDirectory(this,
                                tr("Find Files"), QDir::currentPath());

     if (!directory.isEmpty()) {
         if (ui->directoryComboBox->findText(directory) == -1)
             ui->directoryComboBox->addItem(directory);
         ui->directoryComboBox->setCurrentIndex(ui->directoryComboBox->findText(directory));
     }
}

void FindFile::xupdateComboBox(QComboBox *a_comboBox)
{
    if (a_comboBox->findText(a_comboBox->currentText()) == -1 && !a_comboBox->currentText().isEmpty())
        a_comboBox->addItem(a_comboBox->currentText());
}

void FindFile::on_findButton_clicked()
{
    ui->filesTable->setRowCount(0);

    QString fileName = ui->fileComboBox->currentText();
    QString text = ui->textComboBox->currentText();
    QString path = ui->directoryComboBox->currentText();

    xupdateComboBox(ui->fileComboBox);
    xupdateComboBox(ui->textComboBox);

    currentDir = QDir(path);
    QStringList files;
    if (fileName.isEmpty())
        fileName = "*";
    files = currentDir.entryList(QStringList(fileName),
                                 QDir::Files | QDir::NoSymLinks);

    if (!text.isEmpty())
        files = findFiles(files, text);
    showFiles(files);
}

QStringList FindFile::findFiles(const QStringList &files, const QString &text)
 {
     QProgressDialog progressDialog(this);
     progressDialog.setCancelButtonText(tr("&Cancel"));
     progressDialog.setRange(0, files.size());
     progressDialog.setWindowTitle(tr("Find Files"));

     QStringList foundFiles;

     for (int i = 0; i < files.size(); ++i) {
         progressDialog.setValue(i);
         progressDialog.setLabelText(tr("Searching file number %1 of %2...")
                                     .arg(i).arg(files.size()));
         qApp->processEvents();

         if (progressDialog.wasCanceled())
             break;

         QFile file(currentDir.absoluteFilePath(files[i]));

         if (file.open(QIODevice::ReadOnly)) {
             QString line;
             QTextStream in(&file);
             while (!in.atEnd()) {
                 if (progressDialog.wasCanceled())
                     break;
                 line = in.readLine();
                 if (line.contains(text)) {
                     foundFiles << files[i];
                     break;
                 }
             }
         }
     }
     return foundFiles;
}

void FindFile::showFiles(const QStringList &files)
{
    ui->filesTable->horizontalHeader()->setResizeMode(0, QHeaderView::Stretch);
    ui->filesTable->verticalHeader()->hide();
    ui->filesTable->setShowGrid(false);

    for (int i = 0; i < files.size(); ++i) {
         QFile file(currentDir.absoluteFilePath(files[i]));
         qint64 size = QFileInfo(file).size();

         QTableWidgetItem *fileNameItem = new QTableWidgetItem(files[i]);
         fileNameItem->setFlags(fileNameItem->flags() ^ Qt::ItemIsEditable);
         QTableWidgetItem *sizeItem = new QTableWidgetItem(tr("%1 KB")
                                              .arg(int((size + 1023) / 1024)));
         sizeItem->setTextAlignment(Qt::AlignRight | Qt::AlignVCenter);
         sizeItem->setFlags(sizeItem->flags() ^ Qt::ItemIsEditable);

         int row = ui->filesTable->rowCount();
         ui->filesTable->insertRow(row);
         ui->filesTable->setItem(row, 0, fileNameItem);
         ui->filesTable->setItem(row, 1, sizeItem);
    }
    QString a_str;

    ui->filesFoundLabel->setText(tr("%1 soubor(y) nalezeny").arg(files.size()) +
                              (a_str.fromLocal8Bit(" (Dvojklikem otevete soubor)")));
}

void FindFile::on_filesTable_cellActivated(int row, int column)
{
    QTableWidgetItem *item = ui->filesTable->item(row, 0);

    QString a_adress("file:///");

    a_adress.append(currentDir.absoluteFilePath(item->text()));

    QDesktopServices::openUrl(QUrl(a_adress, QUrl::TolerantMode));
}
